# -*- coding: utf-8 -*-
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.event import notify
from zope.interface import classProvides
from zope.interface import implements
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import ObjectModifiedEvent


class References(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context
        self.deferred_references = {}
        self.reference_fields = options.get('keys') or 'regexp:(.*[Dd]ate)$'

    def __iter__(self):
        for item in self.previous:
            for fieldname in self.reference_fields:
                if item.get(fieldname, ''):
                    references = self.deferred_references.get(fieldname, [])
                    references.append((item['_path'], item[fieldname]))
                    self.deferred_references[fieldname] = references

            yield item

        intids = getUtility(IIntIds)
        for fieldname, references in self.deferred_references.items():
            for path, uid in references:
                items = []
                try:
                    referencer_object = self.context.restrictedTraverse(
                        str(path)
                    )
                except KeyError:
                    continue

                referenced_brains = self.context.portal_catalog(UID=uid)
                if referenced_brains:
                    referenced_brain = referenced_brains[0].getObject()
                    try:
                        to_id = intids.getId(referenced_brain)
                    except KeyError:
                        to_id = intids.register(referenced_brain)
                    items.append(RelationValue(to_id))

                    setattr(referencer_object, fieldname, items[0])
                    notify(ObjectModifiedEvent(referencer_object))



# -*- coding: utf-8 -*-
from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import is_string_in_file
from bobtemplates.plone.base import set_global_vars as base_set_global_vars
from bobtemplates.plone.base import update_file


def _update_setup_py(configurator):
    file_name = u'setup.py'
    file_path = configurator.variables['package.root_folder'] + '/' + file_name
    match_str = '-*- Extra requirements: -*-'
    insert_strings = [
        'collective.transmogrifier',
        'plone.app.transmogrifier',
        'transmogrify.dexterity',
        'ftw.blueprints',
        'collective.jsonmigrator',
    ]
    for insert_str in insert_strings:
        insert_str = "        '{0}',\n".format(insert_str)
        if is_string_in_file(configurator, file_path, insert_str):
            continue
        update_file(configurator, file_path, match_str, insert_str)

    match_str = 'target = plone'
    insert_strings = [
        'run_migration = {0}.migration.scripts:run_migration'.format(
            configurator.variables['package.dottedname']
        ),
        'list_migrations = {0}.migration.scripts:list_migrations'.format(
            configurator.variables['package.dottedname']
        ),
        '[zopectl.command]',
    ]
    for insert_str in insert_strings:
        insert_str = '    {0}\n'.format(insert_str)
        if is_string_in_file(configurator, file_path, insert_str):
            continue
        update_file(configurator, file_path, match_str, insert_str)


def _update_configure_zcml(configurator):
    file_name = u'configure.zcml'
    file_path = configurator.variables['package_folder'] + '/' + file_name

    match_str = '-*- extra stuff goes here -*-'
    insert_str = """
  <include package=".blueprints" />
  <include package=".pipelines" />
"""
    update_file(configurator, file_path, match_str, insert_str)


def prepare_renderer(configurator):
    configurator = base_prepare_renderer(configurator)
    configurator.target_directory = configurator.variables['package_folder']


def post_renderer(configurator):
    _update_setup_py(configurator)
    _update_configure_zcml(configurator)
    git_commit(configurator, 'Added migration')


def set_global_vars(configurator):
    base_set_global_vars(configurator)



# -*- coding: utf-8 -*-
"""
Original: https://gist.github.com/gyst/c3739160a18cf52c93e0132da511b2ee

see https://stackoverflow.com/questions/4008960/pytz-and-etc-gmt-5
for GMT+ and GMT- bug
"""
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import Condition
from collective.transmogrifier.utils import Matcher
from zope.interface import classProvides
from zope.interface import implements

import logging


logger = logging.getLogger(__name__)


class TimezoneFixerSection(object):
    """
    Fixes UnknownTimeZoneError: (UnknownTimeZoneError('GMT+1',),
    by replacing the (unknown) GMT+x timezone with Etc/GMT+x
    """

    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        keys = options.get('keys') or 'regexp:(.*[Dd]ate)$'
        self.keys = Matcher(*keys.splitlines())
        self.condition = Condition(
            options.get('condition', 'python:True'),
            transmogrifier,
            name,
            options,
        )
        self.previous = previous

    def __iter__(self):
        for item in self.previous:
            if self.condition(item):
                for key in item.keys():
                    match = self.keys(key)[1]
                    if match:
                        oldvalue = item[key]
                        if 'GMT+' in item[key]:
                            item[key] = item[key].replace(
                                ' GMT+', ' Etc/GMT-'
                            )  # noqa
                        elif 'GMT-' in item[key]:
                            item[key] = item[key].replace(
                                ' GMT-', ' Etc/GMT+'
                            )  # noqa
                        elif 'GMT' in item[key]:
                            item[key] = item[key].replace(' GMT', ' Etc/GMT')

                        newvalue = item[key]
                        if oldvalue != newvalue:
                            logger.info(
                                'pytz_fixer: field: {}, oldvalue {} newvalue {}'.format(  # noqa
                                    key, oldvalue, newvalue
                                )
                            )
            yield item



# -*- coding: utf-8 -*-


def prepare_renderer(configurator):
    configurator.variables['template_id'] = 'cs_plone_buildout'
    zope_port_number = int(configurator.variables['buildout_zope_port_number'])
    configurator.variables['zope_instance2_port_number'] = zope_port_number + 1
    configurator.variables['zope_instance3_port_number'] = zope_port_number + 2
    configurator.variables['zope_instance4_port_number'] = zope_port_number + 3
    configurator.variables['zope_instance5_port_number'] = zope_port_number + 4
    configurator.variables['zeo_port_number'] = zope_port_number + 10
    configurator.variables['haproxy_port_number'] = zope_port_number + 20
    configurator.variables['varnish_port_number'] = zope_port_number + 30


def post_renderer(configurator):
    """"""
    pass



# -*- coding: utf-8 -*-


class RegEntry(object):
    def __init__(self):
        self.template = ''
        self.plonecli_alias = ''
        self.depend_on = None
        self.deprecated = False
        self.info = ''


def cs_plone_buildout():
    reg = RegEntry()
    reg.template = 'bobtemplates.cs:cs_plone_buildout'
    reg.plonecli_alias = 'cs_plone_buildout'
    return reg


def cs_migration():
    reg = RegEntry()
    reg.template = 'bobtemplates.cs:cs_migration'
    reg.plonecli_alias = 'cs_migration'
    reg.depend_on = 'plone_addon'
    return reg



# -*- coding: utf-8 -*-

from setuptools import find_packages
from setuptools import setup


version = '1.1'


long_description = '\n\n'.join(
    [open('README.rst').read(), open('CHANGES.rst').read()]
)


setup(
    name='bobtemplates.cs',
    version=version,
    description='Templates for Plone projects by CodeSyntax',
    long_description=long_description,
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Development Status :: 5 - Production/Stable',
        'Topic :: Software Development :: Code Generators',
        'Topic :: Utilities',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
    ],
    keywords='web plone zope skeleton project',
    author='Mikel Larreategi',
    author_email='mlarreategi@codesyntax.com',
    url='https://github.com/codesyntax/bobtemplates.cs',
    license='GPLv2',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['bobtemplates'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'mr.bob',
        'bobtemplates.plone',
        'lxml',
        'stringcase',
    ],
    setup_requires=[],
    tests_require=[],
    extras_require={},
    entry_points={
        'mrbob_templates': [
            'cs_plone_buildout = bobtemplates.cs.bobregistry:cs_plone_buildout'  # NOQA E501
            'cs_migration = bobtemplates.cs.bobregistry:cs_migration'  # NOQA E501
        ]
    },
)



# -*- coding: utf-8 -*-
from datetime import date

import logging


logger = logging.getLogger('bobtemplates.cs')


class SetupCfg(object):
    def __init__(self):
        self.version = None


def set_global_vars(configurator):
    configurator.variables['year'] = date.today().year
    version = configurator.variables.get('plone.version')
    _set_plone_version_variables(configurator, version)


def _set_plone_version_variables(configurator, version):
    version = configurator.variables.get('plone.version', version)
    if not version:
        return
    if 'plone.is_plone5' not in configurator.variables:
        # Find out if it is supposed to be Plone 5.
        if version.startswith('5'):
            configurator.variables['plone.is_plone5'] = True
        else:
            configurator.variables['plone.is_plone5'] = False
    if 'plone.minor_version' not in configurator.variables:
        # extract minor version (4.3)
        # (according to https://plone.org/support/version-support-policy)
        # this is used for the trove classifier in setup.py of the product
        configurator.variables['plone.minor_version'] = \
            '.'.join(version.split('.')[:2])
